/*
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://x2c.lcm.at/
 * $LastChangedRevision: 1623 $
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "ConvFnc_AutoSwitch_FiP32.h"

/* USERCODE-BEGIN:PreProcessor                                                                                        */
/* USERCODE-END:PreProcessor                                                                                          */

extern uint8 MaskParamBuffer[];
extern uint8 ImplParamBuffer[];
extern uint8 SaveFncDataBuffer[];

/** Implementation parameter data structure */
typedef struct {
    int32 Thresh_up;
    int32 Thresh_down;
} AUTOSWITCH_FIP32_IMPL_PARAM;

/** Private prototypes */
static uint8 convertM2I(const AUTOSWITCH_FIP32_MASK_PARAM *maskParam, AUTOSWITCH_FIP32_IMPL_PARAM *impParam);

/**
 * @brief Converts Mask parameters to Implementation parameters.
 *
 * @param[in] maskParam Mask parameters
 * @param[out] impParam Implementation parameters
 *
 * @return Error: zero on success, not zero in case of conversion error
 */
static uint8 convertM2I(const AUTOSWITCH_FIP32_MASK_PARAM *maskParam, AUTOSWITCH_FIP32_IMPL_PARAM *impParam)
{
    uint8 error = (uint8)0;
/* USERCODE-BEGIN:Conversion                                                                                          */
    if ((maskParam->Thresh_up) < (maskParam->Thresh_down))
    {
        error = 1;  /* Threshold level conflict: up value must not be smaller than down value!*/
    }
    else
    {
        impParam->Thresh_up   = getQx32Value((float_CoT)(maskParam->Thresh_up), 31);
        impParam->Thresh_down = getQx32Value((float_CoT)(maskParam->Thresh_down), 31);
    }

/* USERCODE-END:Conversion                                                                                            */
    return (error);
}

/**
 * @brief Load block mask parameter data.
 *
 * @param[in] maskParam Mask parameter data structure
 * @param[out] data Data
 * @param[out] dataLen Data length
 * @param[in] maxSize Maximum Service data length
 *
 * @return Error: zero on success, not zero in case of load- or conversion error
 */
uint8 AutoSwitch_FiP32_LoadMP(const AUTOSWITCH_FIP32_MASK_PARAM *maskParam, uint8 data[], uint16* dataLen, uint16 maxSize)
{
    uint8 error = (uint8)0;
    if ((uint16)16 > maxSize)
    {
        error = (uint8)1;
    }
    else
    {
        data[0] = (uint8)((*(uint64*)&(maskParam->Thresh_up)) & 0x00000000000000FF);
        data[1] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 8) & 0x00000000000000FF);
        data[2] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 16) & 0x00000000000000FF);
        data[3] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 24) & 0x00000000000000FF);
        data[4] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 32) & 0x00000000000000FF);
        data[5] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 40) & 0x00000000000000FF);
        data[6] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 48) & 0x00000000000000FF);
        data[7] = (uint8)((*(uint64*)&(maskParam->Thresh_up) >> 56) & 0x00000000000000FF);
        data[8] = (uint8)((*(uint64*)&(maskParam->Thresh_down)) & 0x00000000000000FF);
        data[9] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 8) & 0x00000000000000FF);
        data[10] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 16) & 0x00000000000000FF);
        data[11] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 24) & 0x00000000000000FF);
        data[12] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 32) & 0x00000000000000FF);
        data[13] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 40) & 0x00000000000000FF);
        data[14] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 48) & 0x00000000000000FF);
        data[15] = (uint8)((*(uint64*)&(maskParam->Thresh_down) >> 56) & 0x00000000000000FF);
        *dataLen = (uint16)16;
    }
    return (error);
}

/**
 * @brief Save block mask parameter data.
 *
 * @param[in] block Pointer to block structure
 * @param[out] maskParam Mask parameter data structure
 * @param[in] data Data
 * @param dataLen Length of mask parameter data stream
 *
 * @return Error: zero on success, not zero in case of save- or conversion error
 */
uint8 AutoSwitch_FiP32_SaveMP(AUTOSWITCH_FIP32 *block, AUTOSWITCH_FIP32_MASK_PARAM *maskParam, const uint8 data[], uint16 dataLen)
{
    uint8 error = (uint8)0;

    if (dataLen != (uint16)16)
    {
        error = (uint8)1;
    }
    else
    {
        uint64 tmp64;
        AUTOSWITCH_FIP32_IMPL_PARAM *implParam = (AUTOSWITCH_FIP32_IMPL_PARAM*)ImplParamBuffer;

        /** - Cache old mask parameters */
        memcpy(MaskParamBuffer, maskParam, sizeof(*maskParam));

        /** - Save new mask parameters */
        tmp64 = (uint64)data[0] + \
            ((uint64)data[1] << 8) + ((uint64)data[2] << 16) + \
            ((uint64)data[3] << 24) + ((uint64)data[4] << 32) + \
            ((uint64)data[5] << 40) + ((uint64)data[6] << 48) + \
            ((uint64)data[7] << 56);
        maskParam->Thresh_up = (float64)(*(float64*)&tmp64);
        tmp64 = (uint64)data[8] + \
            ((uint64)data[9] << 8) + ((uint64)data[10] << 16) + \
            ((uint64)data[11] << 24) + ((uint64)data[12] << 32) + \
            ((uint64)data[13] << 40) + ((uint64)data[14] << 48) + \
            ((uint64)data[15] << 56);
        maskParam->Thresh_down = (float64)(*(float64*)&tmp64);

        /** - Convert mask parameters to implementation parameters */
        error = convertM2I(maskParam, implParam);

        /** - Prepare implementation parameters for save function */
        SaveFncDataBuffer[0] = (uint8)(implParam->Thresh_up & 0x000000FF);
        SaveFncDataBuffer[1] = (uint8)((implParam->Thresh_up >> 8) & 0x000000FF);
        SaveFncDataBuffer[2] = (uint8)((implParam->Thresh_up >> 16) & 0x000000FF);
        SaveFncDataBuffer[3] = (uint8)((implParam->Thresh_up >> 24) & 0x000000FF);
        SaveFncDataBuffer[4] = (uint8)(implParam->Thresh_down & 0x000000FF);
        SaveFncDataBuffer[5] = (uint8)((implParam->Thresh_down >> 8) & 0x000000FF);
        SaveFncDataBuffer[6] = (uint8)((implParam->Thresh_down >> 16) & 0x000000FF);
        SaveFncDataBuffer[7] = (uint8)((implParam->Thresh_down >> 24) & 0x000000FF);

        /** - Execute save function of block */
        error = AutoSwitch_FiP32_Save(block, SaveFncDataBuffer, 8);

        /** - Check for errors during execution of save function */
        if (error != 0)
        {
            /* Restore old mask parameter */
            memcpy(maskParam, MaskParamBuffer, sizeof(*maskParam));
        }

    }
    return (error);
}

/**
 * @brief Initializes Implementation parameters from Mask parameters.
 *
 * @param[out] block Block
 * @param[in] maskParam Mask parameters
 *
 * @return Error: zero on success, not zero in case of initialization error
 */
uint8 AutoSwitch_FiP32_InitMP(AUTOSWITCH_FIP32 *block, const AUTOSWITCH_FIP32_MASK_PARAM *maskParam)
{
    uint8 error = (uint8)0;

    AUTOSWITCH_FIP32_IMPL_PARAM *implParam = (AUTOSWITCH_FIP32_IMPL_PARAM*)ImplParamBuffer;

    /** - Convert mask parameters to implementation parameters */
    error = convertM2I(maskParam, implParam);

    /** - Prepare implementation parameters for save function */
    SaveFncDataBuffer[0] = (uint8)(implParam->Thresh_up & 0x000000FF);
    SaveFncDataBuffer[1] = (uint8)((implParam->Thresh_up >> 8) & 0x000000FF);
    SaveFncDataBuffer[2] = (uint8)((implParam->Thresh_up >> 16) & 0x000000FF);
    SaveFncDataBuffer[3] = (uint8)((implParam->Thresh_up >> 24) & 0x000000FF);
    SaveFncDataBuffer[4] = (uint8)(implParam->Thresh_down & 0x000000FF);
    SaveFncDataBuffer[5] = (uint8)((implParam->Thresh_down >> 8) & 0x000000FF);
    SaveFncDataBuffer[6] = (uint8)((implParam->Thresh_down >> 16) & 0x000000FF);
    SaveFncDataBuffer[7] = (uint8)((implParam->Thresh_down >> 24) & 0x000000FF);

    /** - Execute save function of block */
    error = AutoSwitch_FiP32_Save(block, SaveFncDataBuffer, 8);

    return (error);
}
